Exception: Anchormodel::InvalidKey

Inherits:
RuntimeError
  • Object
show all
Defined in:
lib/anchormodel.rb

Overview

Raised when an anchormodel key is unknown to its class — i.e. no new :that_key call was made on the subclass.

Inherits from RuntimeError so existing rescue RuntimeError blocks remain compatible while allowing the narrower rescue Anchormodel::InvalidKey.